
/* Copyright(C) (2002) (ID - IMAG) <Florent.Morata@imag.fr> */

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

/*! \file analyze.cpp
 * \brief Methods of Analyze class.
 * \author - Florent.Morata@imag.fr
           - Corine.Marchand@imag.fr
	       - Jean-Marc.Vincent@imag.fr
 * \date October 2002.		   
 */

// DOXYGEN comments

/*! \fn void Analyze::open_sample(const string path)
 * \brief reading of a sample file.
 * \param path a string corresponding to the input full path name.
 */

/*! \fn int Analyze::elt_cl(const int i) const 
 * \brief Give the ith generate value.
 */

/*! \fn inline int Analyze::elt_time(const int i) const
 * \brief Give the ith coalesce time.
 */

/*! \fn int Analyze::nb_cl() const
 * \brief Give the class number of the generation.
 *
 * If user simulate without functional, the class number is exactly the dimension of the space state. 
 */

/*! \fn inline int Analyze::size_sample() const
 * \brief Give the sample size.
 */

/*! \fn int Analyze::MaxTau() const
 * \brief Give the largest coalesce time value.
 */

/*! \fn void Analyze::create_cl()
 * \brief Creation of histo_cl vector.
 */

/*! \fn void Analyze::create_time()
 * \brief Creation of histo_time vector.
 */

/*! \fn int Analyze::elt_histo_time(const int i) const
 * \brief Give the apparition number of the class value i.
 */

/*! \fn string Analyze::norm_cl(const string file)
 * \brief Normalization of histo_cl vector.
 *
 * It results a file '.stat_cl'.
 * \param file a string corresponding to the sample name.
 */

/*! \fn string Analyze::norm_time(const string file)
 * \brief Normalization of histo_time vector.
 * 
 * It results a file '.stat_time'.
 * \param file a string corresponding to the sample name.
 */

/*! \fn double Analyze::pi(const int i) const
 * \brief Give the ith estimated probability value.
 */

/*! \fn template <class T> T Analyze::sum(vector<T> V,const int choice)
 * \brief Sum of T vector element.
 *
 * We can calculate \f$ \sum_{i=1}^{size} elt \f$, or \f$ \sum_{i=1}^{size} elt^{2} \f$.
 * \param V a T vector, where T is a generic type.
 * \param choice an integer to specify the kind of sum to execute.
 */

/*! \fn double Analyze::tau(vector<int> V)
 * \brief Give the mean estimator.
 * \param V an integer vector of sample values.
 */

/*! \fn double Analyze::sigma(vector<int> V) 
 * \brief Give the standard deviation estimator.
 * \param V an integer vector of sample values.
 */

/*! \fn double Analyze::error(vector<int> V)
 * \brief Give the relative error.
 * \param V an integer vector of sample values.
 */

/*! \fn ldouble Analyze::int_conf(vector<int> V,int choice)
 * \brief Construction of confidence interval.
 *
 * The choice variable permit to choose lower or upper born of the confidence interval.
 * \param V an integer vector of sample values.
 * \param choice an integer to specify the born.
 */

/*! \fn int Analyze::elt_histo_time(const int i) const
 * \brief Give the ith value of vector histo_time.
 */

#ifndef _ANALYZE_CPP_
#define _ANALYZE_CPP_

#include "analyze.h"

//----------------------------------------------------------------------------------------------------------

void Analyze::open_sample(const string path)
{
  ifstream fp(path.c_str());
  int value1,value2;
  char cursor;        // to forget the PSI header
  
  while(cursor !='!')
    fp >> cursor;
  
  fp >> size;
  
  for(int i = 0; i < size; i++)
    {
      fp >> value1;
      cl.push_back(value1);
      fp >> value2;	
      t.push_back(value2);
    }
  
  fp.close();
		
}

//----------------------------------------------------------------------------------------------------------

int Analyze::nb_cl() const
{
  int maxi = 0;
  for(unsigned int i = 0; i < cl.size();i++)
    if(maxi < cl[i]) maxi = cl[i];
  
  return (maxi + 1);
}

//----------------------------------------------------------------------------------------------------------

int Analyze::MaxTau() const
{
  int maxi = 0;
  for(unsigned int i = 0; i < t.size(); i++)
    if(maxi < t[i]) maxi = t[i];
  
  return (maxi);
  
}

//----------------------------------------------------------------------------------------------------------

void Analyze::create_cl()
{
   for(int i = 0 ; i < nb_cl(); i++)
     histo_cl.push_back(0);
   
   for(int i = 0; i < size; i++) histo_cl[cl[i]]++; 
   
}

//----------------------------------------------------------------------------------------------------------

void Analyze::create_time()
{
  
  for(int i = 0 ; i < MaxTau() + 1; i++) 
    histo_t.push_back(0);
  
  cout << " " << endl;
  for(int i = 0; i < size; i++) histo_t[t[i]]++;
  
}

//----------------------------------------------------------------------------------------------------------

string Analyze::norm_cl(const string file)
{
  string path = file + ".stat_cl";	
  ofstream fp(path.c_str());
  
  for(int i = 0 ; i < nb_cl(); i++)
    {
      fp << i << " " << (double) histo_cl[i] / size << endl;
    }
  
  fp.close();
  
  return (path);
}

//----------------------------------------------------------------------------------------------------------

string Analyze::norm_time(const string file)
{
	
  string path = file + ".stat_time";
  ofstream fp(path.c_str());
  
  for(int i = 0 ; i < MaxTau() + 1; i++)
    {
      fp << i << " " << (double) histo_t[i] / size << endl;
    }
  
  fp.close();
  
  return (path);
}

//----------------------------------------------------------------------------------------------------------

double Analyze::pi(const int i) const
{
  return ((double)histo_cl[i]/size);
}

//----------------------------------------------------------------------------------------------------------

template <class T>
T Analyze::sum(vector<T> V,const int choice)
{
  T compt = 0;
  
  for(int i = 0; i < size; i++)
    {
      switch(choice)
	{
	case 1: compt += V[i]; break;
	case 2: compt += V[i]*V[i]; break;
	default : break;
	}
    }
  
  return (compt);
}

//----------------------------------------------------------------------------------------------------------

double Analyze::tau(vector<int> V)
{
  return ((double) sum(V,1)/V.size());
}

//----------------------------------------------------------------------------------------------------------

double Analyze::sigma(vector<int> V) 
{
  vector<ldouble> tmp;
  
  for(unsigned int i = 0 ; i < V.size(); i++)
    {
      tmp.push_back(fabs(V[i] - tau(V)));
    }
  
  return sqrt(sum(tmp,2) /(V.size() - 1)); 
  
}

//----------------------------------------------------------------------------------------------------------

double Analyze::error(vector<int> V)
{
  return (1.96*(sigma(V)/sqrt((double)V.size())));
}

//----------------------------------------------------------------------------------------------------------

ldouble Analyze::int_conf(vector<int> V,int choice)
{
  double inf = tau(V) - error(V);
  double sup = tau(V) + error(V);
  
  switch(choice)
    {
    case 1: return inf; break;
    case 2: return sup; break;
    default:return 0;  break;
    }
}

//----------------------------------------------------------------------------------------------------------

#endif






