
/* Copyright(C) (2002) <Florent.Morata@imag.fr> */

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

/*! \file fic_gnu.cpp
 * \brief Gnuplot manipulation in C++ code.
 * \author - Florent.Morata@imag.fr
           - Corine.Marchand@imag.fr
	       - Jean-Marc.Vincent@imag.fr
                                                                                
 * \date October 07, 2002                                                                   
 */                                                  

#ifndef _FIC_GNU_CPP_
#define _FIC_GNU_CPP_


#include "fic_gnu.h"

//----------------------------------------------------------------------------------------------------------------


/*! \fn void appel_script(string fic)
 * \brief Execution of GnuPlot script.
 * \param fic a string corresponding to the path name of GnuPlot script to execute.
*/
void appel_script(string fic) 
{
  string command; 
  command = "gnuplot " + fic; 
  
  system(command.c_str()); // shell called
}


//----------------------------------------------------------------------------------------------------------------


/*! \fn string create_courb(string name,int bornl,int bornr,string option,gnu_type gt,string xlabel,string ylabel,bool bezier) 
 * \brief Creation of a script.
 *
 * This script is created in the current PSI directory.
 * \param name a string corresponding to the script name.
 * \param bornl an integer corresponding to the lower born.
 * \param bornr an integer corresponding to the upper born.
 * \param option a string to specify style option.
 * \param gt gnu_type corresponding to the exportation extension. 
 * \param xlabel a string to specify x axis name.
 * \param ylabel a string to specify y axis name.
 * \param bezier a boolean to activate 'bezier' option.
 * \return A string corresponding to the path script created.
 */
string create_courb(string name,int bornl,int bornr,string option,gnu_type gt,string xlabel,string ylabel,bool bezier) 
{
  string fic = name + ".gnu";
  string term,ext;
  ofstream in(fic.c_str());
  
  in << "##################################" << endl;
  in << "# GnuPlot script " << endl;
  in << "#    file '" << name << "'  #" << endl;
  in << " " << endl;
  in << " " << endl;
  in << " set encoding iso_8859_1 " << endl;     // Codage des caractres latins
  in << " set title 'File " << name << "'" << endl;
  in << " set xlabel '" << xlabel << "' " << endl;
  in << " set ylabel '" << ylabel << "' " << endl;
  
  
  switch(gt)
    {
    case eps_file : term = "postscript eps color"; ext = "eps"; 
      break;
    case ps_file :  term = "postscript color"; ext = "ps";
      break;
    case jpg_file : term = "jpeg"; ext = "jpg";
      break;
    case tex_file : term = "latex"; ext = "tex";
      break;
    case fig_file : term = "fig"; ext = "fig";
      break;
    case pstex_file : term = "pslatex"; ext = "pstex";
      break;
    }

  
  in << " set terminal " << term << endl; 
  in << " set output '" << name << "." << ext << "'" << endl;
  in << " set nokey " << endl;
  
  if(bezier == true) in << " plot [" << bornl <<  ":" << bornr <<"][*:*]" << "'" << name << "' smooth bezier w " << option << " 3" << endl;
  else in << " plot [" << bornl <<  ":" << bornr <<"][0:*]" << "'" << name << "' w " << option << " 3" << endl;
  
  in.close();
  
  return(fic);
  
}

//----------------------------------------------------------------------------------------------------------------


/*! \fn string create_aff(string name,int bornl,int bornr,string option,string xlabel,string ylabel,bool bezier)
 * \brief  Creation of a posting script.
 * \param name a string corresponding to the script name.
 * \param bornl an integer corresponding to the lower born.
 * \param bornr an integer corresponding to the upper born.
 * \param option a string to specify style option.
 * \param xlabel a string to specify x axis name.
 * \param ylabel a string to specify y axis name.
 * \param bezier a boolean to activate 'bezier' option.
 * \return A string corresponding to the path script created.
 */
string create_aff(string name,int bornl,int bornr,string option,string xlabel,string ylabel,bool bezier)
{
  string fic = name + "_aff.gnu";
  ofstream in(fic.c_str());

  in <<  "#############################" << endl;
  in <<  "# GnuPlot script " << endl;
  in <<  "#        file '" << name << "' " <<  endl;
  in << " " << endl;
  in << " " << endl;
  in << " set encoding iso_8859_1 " << endl;  // Codage des caractres latins
  in << " set title 'File " << name << "'" << endl;
  in << " set xlabel '" << xlabel << "' " << endl;
  in << " set ylabel '" << ylabel << "' " << endl;
  in << " set nokey " << endl;
  
  if(bezier == true) in << " plot [" << bornl <<  ":" << bornr <<"][*:*]" << "'" << name << "' smooth bezier  w " << option << " 3" << endl;
  else in << " plot [" << bornl <<  ":" << bornr <<"][0:*]" << "'" << name << "' w " << option << " 3" << endl;
  
  in << " pause -1  \"  Press enter to continue ... \" " << endl;
  in.close();
  
  return (fic);
}

//----------------------------------------------------------------------------------------------------------------


/*! \fn gnu_type choice_export()
 * \brief User choice of extension exportation file.
 *
 * Actually, available extensions are : 
 - eps.
 - fig.
 - tex.
 - jpg.
 - ps.
 - pstex.
 * \return The file type for GnuPlot exportation.
 */
gnu_type choice_export()
{
  int ext;
  gnu_type gt;	
  cout << " " << endl;
  cout << "  - Exportation : (1) eps / (2) fig / (3) tex / (4) jpeg / (5) ps / (6) pstex "; 
  cin >> ext;
  cout << " " << endl;
  
  switch(ext)
    {	
    case 1: gt = eps_file; break;
    case 2: gt = fig_file; break;
    case 3: gt = tex_file; break;
    case 4: gt = jpg_file; break;
    case 5: gt = ps_file; break;
    case 6: gt = pstex_file; break;
    }
	
  return (gt);
}

//----------------------------------------------------------------------------------------------------------------

#endif

