
/* Copyright(C) (2002) (ID - IMAG) <Florent.Morata@imag.fr> 

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 


/*! \file psi.cpp
 * \brief Main of PSI.
 *
 * This is the textual interface version of PSI.
 * \author - Florent.Morata@imag.fr
           - Corine.Marchand@imag.fr
	       - Jean-Marc.Vincent@imag.fr
 * \date  November 27, 2002.
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


// C++ includes
#include <iostream>
#include <fstream>
#include <string>
#include <iomanip>


/*! \namespace std
 * \brief Utilization of the Standard Template Library.
 */
using namespace std;

// Unix includes
#include <unistd.h>
#include <sys/time.h>


// local includes
#include "hbf.h"
#include "alias.h"
#include "analyze.h"
#include "cost.h"
#include "alea.h"
#include "fic_gnu.h"
#include "interface.h"
#include "percent.h"


//--------------------------------------------------------------------------------------------------------------------------


/*! \fn void precomputation()
 * \brief Precomputation of PSI. 
 */
void precomputation()
{
  Alias A;
  struct timeval beg_const,end_const,beg_read,end_read,beg_wr,end_wr; 
  string path_model;
  bool continu = false;
  
  cout << "\n\n  #### PRECOMPUTATION ####" << endl;
  cout << " " << endl;
  
  while(! continu)          // verify if input path exists
  {
   cout << " - Input model (full path) : " ;
   cin >> path_model;
   continu = ExistsFile(path_model);
  }  
  cout << " " << endl;
 
  cout << "  Data loading ... " << endl;
  gettimeofday(&beg_read,NULL); // determination of input protocole 
  OpenModel(path_model);
  gettimeofday(&end_read,NULL); 
  cout << "   ;-) HBF data now in memory ! " << endl;   

  if (HBF::hbf.type() == continuous) // uniformization
    {
      cout << "\n   Uniformization ... " << endl; 
      HBF::hbf.uniformization();
    } 
  
  cout << "\n    Aliasing arrays in construction ... " << endl;
  cout << " " << endl;
  gettimeofday(&beg_const,NULL); // beginning of aliasing construction
  HBF::hbf.transform(HBF::hbf.out_degree()); // format dimension*dmax
  A.init(HBF::hbf.out_degree()); // first step of aliasing construction
  A.mk_walker(HBF::hbf.out_degree()); // Walker's construction
  A.transpose(HBF::hbf.out_degree());  // transposition of tables
  gettimeofday(&end_const,NULL); // end of aliasing construction
  
  gettimeofday(&beg_wr,NULL);
  A.write_construct(path_model.substr(0,path_model.find('.')),HBF::hbf.out_degree()); // writing of aliasing arrays (by column)
  gettimeofday(&end_wr,NULL); 

  cout << "         :-) File '" << path_model.substr(0,path_model.find('.')) << ".simu' created ...\n\n";  
  cout << "     :-) Aliasing construction time : " << (end_const.tv_sec - beg_const.tv_sec) * 1000000 + (end_const.tv_usec - beg_const.tv_usec) << " micro-seconds" << endl;
  cout << "     :-) Reading file time : " << (end_read.tv_sec - beg_read.tv_sec) * 1000000 + (end_read.tv_usec - beg_read.tv_usec) << " micro-seconds" << endl;
  cout << "     :-) Writing file time : " << (end_wr.tv_sec - beg_wr.tv_sec) * 1000000 + (end_wr.tv_usec - beg_wr.tv_usec) << " micro-seconds" << endl;
}

//--------------------------------------------------------------------------------------------------------------------------


/*! void backward()
 * \brief PSI Exact Simulation. 
 *
 * This is the real kernel simulation of PSI.
 */
void backward()
{
  Alias A;
  Cost C;
  Percent P;
  int size,stop_time;
  struct timeval ti,to,beg_read,end_read;  
  ofstream out;
  string path_in;
  string path_out;
  string path;
  char rep;
  string path_cost;
  bool continu = false;
  
  cout << "\n\n   #### EXACT SIMULATION ####"<< endl;
  cout << " " << endl;
  
  while(! continu)  // verify if 'simu' file exists
  {
   cout << " - Input 'simu' file (full path) : " ;
   cin >> path_in;
   continu = ExistsFile(path_in);
  }
	  
  cout << " " << endl;
  cout << " - Sample's size : " ;
  cin >> size;
  cout << " " << endl;

  continu = false;
  
  while(! continu) // test if output file already exists
  {
   cout << " - Output file (automatically generate with 'sample' extension) : " ;
   cin >> path_out;
   cout << " " << endl;
   path = path_out + ".sample"; // output 'sample' file  
   continu = Overwrite(path);
  }
  
  cout << " " << endl;
  out.open(path.c_str());

  // aliasing data reading
  cout << "   Aliasing data loading ... " << endl;
  gettimeofday(&beg_read,NULL);
  int D = A.read(path_in);     
  gettimeofday(&end_read,NULL);
  cout << "\n        ;-) Aliasing data now in memory ! " << endl ;
  cout << " " << endl;

  int N = A.size() / D; // recuperation of dimension

  cout << "   - Do you want functional simulation ? (y/n) : " ;
  cin >> rep;
  cout << " " << endl;

  if (rep =='y' || rep == 'Y') {
    cout << "      - Cost file (full path) :";
    cin >> path_cost;
    C.mk_cost(path_cost); 
  }
  
  else C.init_state(N);
  
  cout << " " << endl;
  
  out << "################################################### " << endl;
  out << "#                PSI 2002 (backward simulation)   # " << endl;
  out << "#                                                 # " << endl;
  out << "#              sample's size                      # " << endl;
  out << "#                                                 # " << endl;
  out << "#        Cost + iterations                        # " << endl;
  out << "##################################################! " << endl;
  out << " " << endl;
  
  init_generator(); // initialization of random generator
  out <<  size << endl;
  out << " " << endl;

  int d = 0; // simulation time
  
  gettimeofday(&ti,NULL);
  cout << "           SIMULATION PROGRESS : " << endl;

  while(d != size)
    {
      C.init();
      stop_time = 0;
      
      do
	{
	  stop_time++;

	  double rand = uniform();   // Random called
	  int j = (int) (D*uniform());
	  
	  for(int i = 0; i < N; i++)         
	    C.change(i,A.walk(rand,j,N,D,i)); // Aliasing generation of (backward) trajectories
	  
	  C.swap(); // cost vectors update
	  
	} while(C.test() == true);
      
      out << C.value() << " " << stop_time << endl;
      
      // Percent progress
      P.create(size,d);
      P.progress();
      P.update();

      C.free_cl();
      d++;
      
    }

  cout << "100%" << endl;
  cout << " " << endl;
  cout << " " << endl;
  gettimeofday(&to,NULL);
  
  out.close(); 
  cout << "       :-) File '" << path << "' created ...\n\n";	
  
  // simulation times	
  cout << "    :-) Simulation of one realization : " <<  ((to.tv_sec-ti.tv_sec)*1000000 + (to.tv_usec-ti.tv_usec)) / size << " micro-seconds" << endl;
  cout << "    :-) Reading aliasing file time : " << ((end_read.tv_sec-beg_read.tv_sec)*1000000 + (end_read.tv_usec-beg_read.tv_usec)) / size << " micro-seconds" << endl; 
  cout << " " << endl;
}

//--------------------------------------------------------------------------------------------------------------------------


/*! \fn void forward()
 * \brief PSI Forward simulation.
 *
 * This simulation technique is based on the same coupling technique than Perfect Simulation, but it returns a not exact
 * sampling of \f$ \pi \f$.
 */
void forward()
{
  Alias A;
  Cost C;
  Percent P;
  int size,stop_time;
  struct timeval ti,to,beg_read,end_read;  
  ofstream out;
  string path_in;
  string path_out;
  string path;
  char rep;
  string path_cost;
  bool continu = false;
  
  cout << "\n\n   #### FORWARD SIMULATION ####"<< endl;
  cout << " " << endl;
  while(! continu) // verify if input 'simu' file exists
  {
   cout << " - Input 'simu' file (full path) : " ;
   cin >> path_in;
   continu = ExistsFile(path_in);
  }
  cout << " " << endl;
  
  cout << " - Sample's size : " ;
  cin >> size;
  cout << " " << endl;
  
  continu = false;
  
  while(! continu) // verify if output file already exists
  {
   cout << " - Output file (automatically generate with 'sample' extension) : " ;
   cin >> path_out;
   path = path_out + ".sample"; // output 'sample' file  
   cout << " " << endl;	  
   continu = Overwrite(path);
  }
  
  cout << " " << endl;
  out.open(path.c_str());

  // aliasing data reading
  cout << "   Aliasing data loading ... " << endl;
  gettimeofday(&beg_read,NULL);
  int D = A.read(path_in);     
  gettimeofday(&end_read,NULL);
  cout << "\n        ;-) Aliasing data now in memory ! " << endl ;
  cout << " " << endl;

  int N = A.size() / D; // recuperation of dimension

  cout << "   - Do you want functional simulation ? (y/n) : " ;
  cin >> rep;
  cout << " " << endl;

  if (rep =='y' || rep == 'Y') {
    cout << "      - Cost file (full path) :";
    cin >> path_cost;
    C.mk_cost(path_cost); 
  }
  
  else C.init_state(N);

  cout << " " << endl;
  
  out << "################################################### " << endl;
  out << "#                PSI 2002 (forward simulation)    # " << endl;
  out << "#                                                 # " << endl;
  out << "#              sample's size                      # " << endl;
  out << "#                                                 # " << endl;
  out << "#        Cost + iterations                        # " << endl;
  out << "##################################################! " << endl;
  out << " " << endl;
  
  init_generator(); // initialization of random generator
  out <<  size << endl;
  out << " " << endl;

  int d = 0; // simulation time
  gettimeofday(&ti,NULL);
  cout << "           SIMULATION PROGRESS : " << endl;
  
  while(d != size)
    {
      C.init();
      stop_time = 0;
      
      do
	{
	  stop_time++;
	  
	  double rand = uniform();     // Random called
	  int j = (int) (D*uniform());
	  
	  for(int i = 0; i < N; i++)         
	    C.change(i,A.walk(rand,j,N,D,C.value_next(i)));  // aliasing generation of (forward) trajectories
	  
	} while(C.test_next() == true);
      
      out << C.value_next() << " " << stop_time << endl;
      
      // Percent progress
      P.create(size,d);
      P.progress();
      P.update();

      C.free_cl();
      d++;
    }
  
  gettimeofday(&to,NULL);
  cout << "100%" << endl;  
  out.close();
  cout << " " << endl; 	
  cout << "       :-) File '" << path << "' created ...\n\n";
  
  // simulation times	
  cout << "    :-) Simulation of one realization : " <<  ((to.tv_sec-ti.tv_sec)*1000000 + (to.tv_usec-ti.tv_usec)) / size << " micro-seconds" << endl;
  cout << "    :-) Reading aliasing file time : " << ((end_read.tv_sec-beg_read.tv_sec)*1000000 + (end_read.tv_usec-beg_read.tv_usec)) / size << " micro-seconds" << endl; 
  cout << " " << endl;
	
}

//--------------------------------------------------------------------------------------------------------------------------


/*! \fn void mcmc()
 * \brief Classical Monte Carlo Simulation.
 *
 * We give an arbitrary initial state and a steps number for Monte Carlo simulation.
 * This steps number correspond to the burn-in time wished by the user.
 */
void mcmc()
{
  Alias A;
  Cost C;
  Percent P;
  int size,stop_time,state,duration;
  struct timeval ti,to,beg_read,end_read;  
  ofstream out;
  string path_in;
  string path_out;
  string path;
  char rep;
  string path_cost;
  bool continu = false;
  
  cout << "\n\n   #### MCMC SIMULATION ####"<< endl;
  cout << " " << endl;
  while(! continu) // verify if input 'simu' file exists
  {
   cout << " - Input 'simu' file (full path) : " ;
   cin >> path_in;
   continu = ExistsFile(path_in);
  }

  cout << " " << endl;
  cout << " - Sample's size : " ;
  cin >> size;  
  cout << " " << endl;
  
  continu = false;
  
  while(! continu) // verify if output file already exists
  {
   cout << " - Output file (automatically generate with 'sample' extension): " ;
   cin >> path_out;
   path = path_out + ".sample"; // output 'sample' file
   cout << " " << endl;
   continu = Overwrite(path);
  }
	  
  cout << " " << endl;
  out.open(path.c_str());

  // aliasing data reading
  gettimeofday(&beg_read,NULL);
  cout << "   Aliasing data loading ... " << endl;
  int D = A.read(path_in);     
  gettimeofday(&end_read,NULL);
  cout << "\n        ;-) Aliasing data now in memory ! " << endl ;
  cout << " " << endl;
  
  int N = A.size() / D; // recuperation of dimension

  cout << "   - Do you want functional simulation ? (y/n) : " ;
  cin >> rep;
  cout << " " << endl;

  if (rep =='y' || rep == 'Y') {
    cout << "      - Cost file (full path) :";
    cin >> path_cost;
    C.mk_cost(path_cost); 
  }
  
  else C.init_state(N);

  cout << " " << endl;
  out << "################################################### " << endl;
  out << "#                PSI 2002 (MCMC simulation)       # " << endl;
  out << "#                                                 # " << endl;
  out << "#              sample's size                      # " << endl;
  out << "#                                                 # " << endl;
  out << "#        Cost + burn-in time                      # " << endl;
  out << "##################################################! " << endl;
  out << " " << endl;
  
  cout << "    - Initial state number (0 .. " << N-1 << ") : "; // arbitrary initial state
  cin >> state;
  cout << " " << endl;
  cout << "    - Steps number of simulation : ";
  cin >> duration;
  cout << " " << endl;
  
  init_generator(); // initialization of random generator
  out <<  size << endl;
  out << " " << endl;

  int i = 0;
  int tmp = state;
  gettimeofday(&ti,NULL);
  cout << "           SIMULATION PROGRESS : " << endl;
  
  while(i != size)
    {
      C.init();
      stop_time = 0;
      
      do
	{
	  stop_time++;
	  
	  double rand = uniform();   // Random called
	  int j = (int) (D*uniform());
	  
	  state = C.value(A.walk(rand,j,N,D,state));	
	  
	} while(stop_time < duration);
      
      out << state << " " << stop_time << endl;

      // Percent progress
      P.create(size,i);
      P.progress();
      P.update();  
      
      C.free_cl();
      state = tmp;
      i++;
      
    }
  
  gettimeofday(&to,NULL);
  cout << "100%" << endl;
  out.close(); 	
  cout << " " << endl;
  cout << "       :-) File '" << path << "' created ...\n\n";
  
  // simulation times	
  cout << "    :-) Simulation of one realization : " <<  ((to.tv_sec-ti.tv_sec)*1000000 + (to.tv_usec-ti.tv_usec)) / size << " micro-seconds" << endl;
  cout << "    :-) Reading aliasing file time : " << ((end_read.tv_sec-beg_read.tv_sec)*1000000 + (end_read.tv_usec-beg_read.tv_usec)) / size << " micro-seconds" << endl; 
  cout << " " << endl;
}

//--------------------------------------------------------------------------------------------------------------------------

/*! \fn void trajectory()
 * \brief Give a forward trajectory.
 *
 * User gives initial state and length of trajectory. Results can be seen in corresponding '.traj' file.
 */
void trajectory()
{
  Alias A;
  Cost C;
  Percent P;
  string path_in,path_out,path_cost,path; 
  int state,length,stop_time;
  bool continu = false;
  ofstream out;
  char rep;
		 
  cout << "\n\n   #### TRAJECTORY SIMULATION ####"<< endl;
  cout << " " << endl;
  while(! continu) // verify if input 'simu' file exists
  {
   cout << " - Input 'simu' file (full path) : " ;
   cin >> path_in;
   continu = ExistsFile(path_in);
  }
  
  cout << " " << endl;
  continu = false;
  
  while(! continu) // verify if output file already exists
  {
   cout << " - Ouput file (automatically generate with 'traj' extension) : " ;
   cin >> path_out;
   path = path_out + ".traj"; // output 'sample' file
   cout << " " << endl;
   continu = Overwrite(path);
  }
	 
  cout << " " << endl; 
  out.open(path.c_str());

  // aliasing data reading
  cout << "   Aliasing data loading ... " << endl;
  int D = A.read(path_in);      
  cout << "\n        ;-) Aliasing data now in memory ! " << endl ;
  cout << " " << endl;

  int N = A.size() / D; // recuperation of dimension
  
  cout << "   - Do you want functional simulation ? (y/n) : " ;
  cin >> rep;
  cout << " " << endl;

  if (rep =='y' || rep == 'Y') {
    cout << "      - Cost file (full path) :";
    cin >> path_cost;
    C.mk_cost(path_cost); 
  }
  
  else C.init_state(N);
  
  cout << "   - Initial state ( 0 .. " << N-1 << ") : ";
  cin >> state;
  cout << " " << endl;
  cout << "   - Length of trajectory : ";
  cin >> length;
  cout << " " << endl;
  
  out << "################################################### " << endl;
  out << "#                PSI 2002                         # " << endl;
  out << "#                                                 # " << endl;
  out << "#            Trajectory simulation                # " << endl;
  out << "#                                                 # " << endl;
  out << "#   cost + reject threshold + column index        # " << endl;
  out << "################################################### " << endl;
  out << " " << endl;
  out << " " << endl;
  out << " Initial state : " << state << endl;
  out << " Length of trajectory : " << length << endl;
  out << " " << endl;
  
  stop_time = 0;
  C.init();
  cout << "           SIMULATION PROGRESS : " << endl;

  do
    {
      stop_time++;
      double rand = uniform();   // Random called
      int j = (int) (D*uniform());
      state = C.value(A.walk(rand,j,N,D,state));	
      out << state << " " << rand << " " << j << endl;
      P.create(length,stop_time);  // Percent progress
      P.progress();
      P.update();

    } while(stop_time < length);
  
  C.free_cl();
  out.close();
  cout << " " << endl;
  cout << "\n       :-) Final state : " << state << endl;
  cout << "\n       :-) See '" << path << "' file for complete trajectory. ";
  cout << " " << endl;
  
}

//--------------------------------------------------------------------------------------------------------------------------


/*! \n void norm()
 * \brief Graphical analyze.
 *
 * We obtain graphics of stationary distribution estimation  (or functional) and iterations number observation. 
 */
void norm()
{
  string model;
  Analyze a;
  bool continu = false;
	
  cout << "\n\n #### GRAPHICAL ANALYZE ####" << endl;
  cout << " " << endl;
  while(! continu)
  {
   cout << " - Input 'sample' file (full path) : ";
   cin >> model;
   continu = ExistsFile(model);
  }
	 
  cout << " " << endl;			
  a.open_sample(model); // opening of sample file
  
  a.create_cl();   // creation of normalized vector
  a.create_time();
 
  appel_script(create_aff(a.norm_time(model),0,a.MaxTau(),"lines","Iterations","Probability estimation",true));
  appel_script(create_courb(a.norm_time(model),0,a.MaxTau(),"lines",choice_export(),"Iterations","Probability estimation",true));
  cout << " " << endl;
  
  appel_script(create_aff(a.norm_cl(model),-1,a.nb_cl(),"boxes","States","Probability estimation",false));
  appel_script(create_courb(a.norm_cl(model),-1,a.nb_cl(),"boxes",choice_export(),"States","Probability estimation",false)); 
  
} 

//--------------------------------------------------------------------------------------------------------------------------


/*! \fn void simu()
 * \brief PSI Simulation Menu.
 *
 * Backward simulation is based on CFTP algorithm and return an exact sampling of \f$ \pi \f$, whereas Monte Carlo simulation
 * return a pseudo sampling of \f$ \pi \f$.
 */
void simu()
{
  int choice;
  cout << "\n\n       #### PSI simulation #### " << endl;
  cout << "\n    (1) Backward    (2) Monte Carlo   (0) Back";
  cout << "\n    (3) Forward     (4) Trajectory " ;
  cin >> choice;
	
  switch(choice)
  {
	  case 1 : backward(); break;  
	  case 2 : mcmc(); break;
	  case 3 : forward(); break;
	  case 4 : trajectory(); break;
  }
	
}

//--------------------------------------------------------------------------------------------------------------------------


/*! \fn void tool()
 * \brief PSI tools.
 */
void tool()
{
  int choice,rep1,rep2;
  cout << " \n\n     #### PSI tools #### " << endl;
  cout << "\n   (1) HBF facilities      (3) Aliasing coding";
  cout << "\n   (2) Results             (0) Back";
  cin >> choice;
  
  switch(choice)
    {
    case 1 : cout << "\n\n #### HBF facilities #### " << endl;
      cout << "\n (1) See model  (3) Uniformization "; 
      cout << "\n (2) Conversion (0) Back " << endl;
      cin >> rep1;
      switch(rep1)
	{
	case 1 : See_model(); break;
	case 2 : Conversion(); break;
	case 3 : Unif(); break;
	}
      break;
      
    case 2 : cout << "\n\n #### Results #### " << endl;
      cout << "\n (1) Statistics  (0) Back ";
      cout << "\n (2) Distribution " ;
      cin >> rep2;
      switch(rep2)
	{
	case 1 : Stat(); break;
	case 2 : See_simu(); break;
	}
      break;
       
    case 3 : Walk(); break;
      
    }
  
}

//--------------------------------------------------------------------------------------------------------------------------


/*! \fn int main(int argc,char **argv)
 * \brief PSI Main Program. 
 *
 * Different interface menus of PSI.
 */	
int main(int argc,char **argv)
{
  bool continu = true;
  
  do
    {
      switch(Menu())
	{
	case pre : precomputation(); break;
	case sim : simu(); break;
	case analy : norm(); break;
	case tools : tool(); break;
	case bye : continu = false; break;
	case credits : Credits(); break;	
	}
      
    }while(continu == true);
  
  Bye();
  
}

//--------------------------------------------------------------------------------------------------------------------------



